# Copyright 2010 Canonical Ltd.
#
# This file is part of desktopcouch.
#
# desktopcouch is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# desktopcouch is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with desktopcouch.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Eric Casteleijn <eric.casteleijn@canonical.com>
#          Vincenzo Di Somma <vincenzo.di.somma@canonical.com>

"""Basic migration infrastructure"""

import logging

from desktopcouch.records import database
from desktopcouch.application import server
from desktopcouch.application.platform import find_port


MIGRATION_DESIGN_DOCUMENT = 'dc_migration'


def _all_dbs(ctx):
    """Get all the non private dbs from a server."""
    port = find_port(ctx=ctx)
    uri = "http://localhost:%s" % port
    couchdb_server = server.DesktopServer(uri, oauth_tokens=None, ctx=ctx)
    return [x for x in couchdb_server if not
            x.startswith('_') and not x == database.DCTRASH]


def register(migration_method, dbs=None):
    """Register a migration script, with the view and the dbs it is meant to
    migrate."""
    if dbs is None:
        dbs = []
    MIGRATIONS_REGISTRY.append(
        {'method': migration_method, 'dbs': dbs})


def run_needed_migrations(ctx=None):
    """Run the actual migration."""
    from desktopcouch.application import local_files
    if ctx is None:
        ctx = local_files.DEFAULT_CONTEXT
    all_dbs = _all_dbs(ctx)
    for migration in MIGRATIONS_REGISTRY:
        dbs = migration['dbs']
        if not dbs:
            dbs = all_dbs
        for db_name in dbs:
            db = server.DesktopDatabase(database=db_name, ctx=ctx)
            try:
                migration['method'](db)
                logging.info("Migrating DB: %s", db)
            except database.NoSuchDatabase:
                pass

DELETION_VIEW_NAME = 'migrate_deleted'

DELETION_VIEW_CODE = """
    function(doc) {
        if (doc['application_annotations']['Ubuntu One']
        ['private_application_annotations']['deleted'] && doc.record_type) {
        emit(doc.id, doc.id);
        }
    }"""


def migrate_from_deleted_to_trash(migrateable_db):
    """Migrate from deleted flag to the dc_trash database."""
    migrateable_db.add_view(
        DELETION_VIEW_NAME, map_js=DELETION_VIEW_CODE,
        design_doc=MIGRATION_DESIGN_DOCUMENT)
    view_result = migrateable_db.execute_view(
        view_name=DELETION_VIEW_NAME, design_doc=MIGRATION_DESIGN_DOCUMENT)
    for result in view_result:
        migrateable_db.delete_record(result.id)


MIGRATIONS_REGISTRY = [{'method': migrate_from_deleted_to_trash, 'dbs': []}]
