"""Tests for Desktop CouchDB"""

import atexit
import os
import unittest
import tempfile
import shutil


from desktopcouch.application import local_files
from desktopcouch.application.start_local_couchdb import start_couchdb
from desktopcouch.application.stop_local_couchdb import stop_couchdb
from desktopcouch.application.util import make_random_string

import gobject
gobject.set_application_name("desktopcouch testing")

import gnomekeyring


class TestCase(unittest.TestCase):
    """Base class for tests."""

    def __init__(self, *args, **kwargs):
        super(TestCase, self).__init__(*args, **kwargs)
        # pylint: disable=C0103
        if not hasattr(self, 'assertIn'):
            self.assertIn = lambda value, container, *args: \
                            self.assertTrue(value in container, *args)
        if not hasattr(self, 'assertNotIn'):
            self.assertNotIn = lambda value, container, *args: \
                               self.assertTrue(value not in container, *args)
        if not hasattr(self, 'assertIs'):
            self.assertIs = lambda value1, value2, *args: \
                            self.assertTrue(value1 is value2, *args)
        if not hasattr(self, 'assertIsNot'):
            self.assertIsNot = lambda value1, value2, *args: \
                               self.assertTrue(value1 is not value2, *args)


class TestSecret(object):
    """Test secret."""
    def __init__(self, secret):
        self.secret = secret


class TestKeyring(object):
    """A gnomekeyring replacement that isolates us from the users keyring."""

    def __init__(self, appear_absent=False):
        self.appear_absent = appear_absent
        self.items = {}

    def _get_key(self, attributes):     # pylint: disable=R0201
        """Get key."""
        return "".join([str(a) for a in sorted(attributes.items())])

    def find_items_sync(self, item_type, attributes):
        """Find items sync."""
        key = self._get_key(attributes)
        if item_type in self.items and key in self.items[item_type]:
            return [TestSecret(self.items[item_type][key])]
        raise gnomekeyring.NoMatchError

    # pylint: disable=W0613
    def item_create_sync(self, keyring, item_type, display_name, attributes,
                         secret, update_if_exists):
        """Item create sync."""
        assert keyring is None, "Mock not coded to have multiple keyrings"
        assert update_if_exists is True, "Mock not coded to create duplicates"
        if self.appear_absent:
            raise gnomekeyring.NoKeyringDaemonError
        self.items.setdefault(item_type, {})
        key = self._get_key(attributes)
        self.items[item_type][key] = secret
        return "Mock keyring doesn't have item ids"
    # pylint: enable=W0613

    def get_user_name_password(self):
        """Return the user name and passwd used to access desktopcouch."""
        admin_username = None
        admin_password = None
        try:
            data = self.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET,
                {'desktopcouch': 'basic'})
            admin_username, admin_password = data[0].secret.split(":", 1)
        except gnomekeyring.NoMatchError:
            # generate the random strings to be used as username and psswd
            admin_username = make_random_string(10)
            admin_password = make_random_string(10)
            try:
                # save admin account details in keyring
                self.item_create_sync(
                    None,
                    gnomekeyring.ITEM_GENERIC_SECRET,
                    'Desktop Couch user authentication',
                    {'desktopcouch': 'basic'},
                    ":".join([admin_username, admin_password]), True)
            except gnomekeyring.NoKeyringDaemonError:
                pass
                ## logging.warn(
                ##     "There is no keyring to store our admin credentials.")
            except gnomekeyring.CancelledError:
                pass
                ## logging.warn(
                ##     "There is no keyring to store our admin credentials.")
        return (admin_username, admin_password)

    def get_oauth_data(self):
        """Return the oauth data used to connect with couchdb."""
        consumer_key = make_random_string(10)
        consumer_secret = make_random_string(10)
        token = make_random_string(10)
        token_secret = make_random_string(10)

        # Save the new OAuth creds so that 3rd-party apps can
        # authenticate by accessing the keyring first.  This is
        # one-way.  We don't read from keyring.

        try:
            self.item_create_sync(
                None,
                gnomekeyring.ITEM_GENERIC_SECRET,
                'Desktop Couch user authentication', {'desktopcouch': 'oauth'},
                ":".join([consumer_key, consumer_secret, token, token_secret]),
                True)
        except gnomekeyring.NoKeyringDaemonError:
            # in this case we do not have a major issue, continue and return
            pass
            ## logging.warn(
            ##     "There is no keyring to store our oauth credentials.")

        except gnomekeyring.CancelledError:
            # in this case we do not have a major issue, continue and return
            pass
            ## logging.warn(
            ##     "There is no keyring to store our admin credentials.")
        return (consumer_key, consumer_secret, token, token_secret)


def create_new_test_environment():
    """Create a new test environment."""
    basedir = tempfile.mkdtemp()
    if not os.path.exists(basedir):
        os.mkdir(basedir)

    cache = os.path.join(basedir, 'cache')
    data = os.path.join(basedir, 'data')
    config = os.path.join(basedir, 'config')
    new_context = local_files.Context(
        cache, data, config, keyring=TestKeyring())
    new_context.couchdb_log_level = 'debug'

    # Add etc folder to config
    source_tree_etc_folder = os.path.realpath(
      os.path.join(os.path.split(__file__)[0], "..", "..", "config"))
    if os.path.isdir(source_tree_etc_folder):
        os.environ["XDG_CONFIG_DIRS"] = source_tree_etc_folder

    def stop_test_couch(temp_dir, ctx):
        """Stop the test couchdatabase."""
        stop_couchdb(ctx)
        shutil.rmtree(temp_dir)

    start_couchdb(ctx=new_context)
    atexit.register(stop_test_couch, basedir, new_context)

    return new_context


# TODO: Remove these after you're sure nothing we care about uses these env.
os.environ['XDG_CACHE_HOME'] = "/cachehome"
os.environ['XDG_DATA_HOME'] = "/datahome"
os.environ['XDG_CONFIG_HOME'] = "/confighome"
os.environ['XDG_DATA_DIRS'] = ''

test_context = create_new_test_environment()  # pylint: disable=C0103
